

#pragma once

#include "CoreMinimal.h"
#include "ClothSimulationComponent.h"
#include "RuntimeMeshActor.h"
#include "GameFramework/Actor.h"
#include "ClothActor.generated.h"

class URuntimeMeshProviderStatic;

UCLASS()
class CV5_LATKA_API AClothActor : public ARuntimeMeshActor
{
	GENERATED_BODY()
	
public:	
	// Sets default values for this actor's properties
	AClothActor();

	// These parameters are passed to the actual simulation component.
	// Mass of each cloth node.
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Simulation parameters")
	float NodeMass = 0.1f;

	// Elastic modulus of the structural springs.
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Simulation parameters")
	float StructuralElasticity = 1.0f;

	// Elastic modulus of the shear springs.
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Simulation parameters")
	float ShearElasticity = 1.0f;

	// Elastic modulus of the flexion springs.
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Simulation parameters")
	float FlexionElasticity = 1.0f;

	// Damping factor - higher damping means higher viscosity.
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Simulation parameters")
	float Damping = 1.0f;

	// Slows down the time step by dividing DeltaTime by this value. This
	// is passed to the simulation every tick.
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Simulation parameters")
	float TimeDilation = 1.0f;

	// Scale of the generated cloth.
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Simulation parameters")
	float Scale = 100.0f;

	// Number of squares in the grid (both width and height).
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Simulation parameters")
	int32 GridSize = 10;

	// Actor for which to compute basic one-sided collision.
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Display and interaction")
	TSubclassOf<AActor> CollisionActor;

	// Material to use on the generated mesh.
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Display and interaction")
	UMaterialInterface* Material;

protected:
	// Generate a grid for RuntimeMeshComponent. Creates vertices and their parameters, and also
	// creates triangle indices. This is saved to MeshData for later use.
	void GenerateGrid();

	// Prepares RuntimeMeshComponent for first data upload, generates the mesh grid using GenerateGrid
	// and uploads the data to RMC.
	void GenerateMesh();

	// Takes current simulation node positions and updates the grid mesh accordingly.
	void UpdateMesh();
	
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;
	
	UPROPERTY(Transient)
	URuntimeMeshProviderStatic* StaticProvider;

	UPROPERTY()
	UClothSimulationComponent* ClothSimulation;

	FRuntimeMeshRenderableMeshData MeshData;
	FRuntimeMeshSectionProperties MeshProperties;
public:	
	// Called every frame
	virtual void Tick(float DeltaTime) override;

};
