

#pragma once

#include "CoreMinimal.h"
#include "ClothUtil.h"

/**
 * Represents a mass node in the mass-spring simulation model.
 */
class CV5_LATKA_API FClothNode
{
public:
	explicit FClothNode(const FVector3f Position, const float Mass = 1.0f) : Position(Position), Mass(Mass) {};
	~FClothNode() {};

	// Tells this node to consider the argument spring when calculating forces acting on it.
	// Does NOT do the actual connection of two points using the spring.
	void AddSpring(FClothSpringRef InSpring);
	
	void Tick(float DeltaTime, UWorld* World, const FTransform& ParentTransform, TSubclassOf<AActor> CollisionType);

	// Set whether the node is able to move - fixed node has forced zero speed.
	void SetFixed(bool bInFixed);

	FVector3f GetPosition() const;
	FVector3f GetSpeed() const;

	// Replaces the current force value with a new one computed from all added springs.
	void RecalculateForce();

	// Adds a singular force value to be included when ticking. Use after RecalculateForce.
	void AddForce(const FVector3f& InForce);
	
private:
	FVector3f Position;
	const float Mass = 1.0f;
	const float CollisionRadius = 100.0f;
	bool bFixed = false;
	
	FVector3f Speed = FVector3f::ZeroVector;
	FVector3f ActingForce = FVector3f::ZeroVector;

	TArray<FClothSpringRef> Springs;

	// Modifies the speed as a result of sphere collision.
	void ChangeSpeedAcordingToIntersectedSphere(const FVector3f& IntersectionPoint, const FVector3f& NodePosition);

	// Simple point-sphere intersection check. Returns true if intersecting.
	bool TestSphereIntersection(const FVector3f& Location, float Radius) const;
};
