#pragma once

#include "CoreMinimal.h"
#include "ClothUtil.h"

/**
 * Represents a spring in the mass-spring simulation model. Can compute the acting force of this spring
 * based on its parameters.
 */
class CV5_LATKA_API FClothSpring
{
public:
	FClothSpring(const FClothNodeRef First, const FClothNodeRef Second, const float Elasticity, const float RestLength,
	             const float DampingFactor)
		: First(First), Second(Second), Elasticity(Elasticity), RestLength(RestLength), DampingFactor(DampingFactor)
	{};

	~FClothSpring() {};

	// Calculates elastic and damping forces for this particular spring. Gets called twice (once from each connected node).
	// The calling node passes itself as a parameter.
	FVector3f GetForce(const FClothNode* ThisNode) const;
private:
	FClothNodeRef First;
	FClothNodeRef Second;
	float Elasticity;
	float RestLength;
	float DampingFactor;
};
